==> tpds_extension_ta010_support-1.5.6-py3-none-any.whl: wpc_auth_firmware/src/app.c <==
/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "cryptoauthlib.h"
#include "atcacert/atcacert_client.h"
#include "atcacert/atcacert_der.h"
#include "atcacert/atcacert_host_hw.h"
#include "../../../ta010_tflxwpc.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************
#define WPC_CHALLENGE_SIZE 32

extern ATCAIfaceCfg ecc204_0_init_data;
extern ATCAIfaceCfg atecc608_0_init_data;

uint8_t g_product_cert[320];
size_t g_product_cert_size;

uint8_t g_pr_challenge[WPC_CHALLENGE_SIZE];

uint8_t g_host_wpc_digest[ATCA_SHA_DIGEST_SIZE];
uint8_t g_device_wpc_digest[ATCA_SHA_DIGEST_SIZE];

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

void print_hex(char *str, uint8_t *hex, size_t size)
{
    char display_str[1024];
    size_t display_len = sizeof(display_str);
    atcab_bin2hex(hex, size, display_str, &display_len);
    printf("\n%s\r\n%s\r\n\r\n", str, display_str);
}

ATCA_STATUS verify_certificate(const uint8_t *cert, const uint8_t *public_key, size_t sign_offset, size_t tbs_offset, size_t tbs_size)
{
    ATCA_STATUS status;
    uint8_t signature[ATCA_SIG_SIZE];
    uint8_t tbs_digest[ATCA_SHA_DIGEST_SIZE];
    size_t der_sig_size = sizeof(cert) - sign_offset;

    do
    {
        // Extract Signature from certificate
        if (ATCA_SUCCESS != (status = atcacert_der_dec_ecdsa_sig_value(&cert[sign_offset], &der_sig_size, signature)))
        {
            printf("atcacert_der_dec_ecdsa_sig_value is failed with : %d\r\n", status);
            break;
        }

        // calculate TBS digest
        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256(&cert[tbs_offset], tbs_size, tbs_digest)))
        {
            printf("Calculate TBS digest - atcac_sw_sha2_256 is failed with : %d\r\n", status);
            break;
        }

        // Verify Root certificate against root public key
        bool is_verified;
        if (ATCA_SUCCESS != (status = atcab_verify_extern(tbs_digest, signature, public_key, &is_verified)))
        {
            printf("atcab_verify_extern is failed with: %d\r\n", status);
            break;
        }
        status = is_verified ? ATCA_SUCCESS : ATCA_GEN_FAIL;

    } while (0);

    return status;
}

ATCA_STATUS calculate_wpc_digests(const uint8_t *root_cert, size_t root_cert_size, const uint8_t *mfg_cert, size_t mfg_cert_size,
                                  const uint8_t *pu_cert, size_t pu_cert_size, uint8_t wpc_digest[ATCA_SHA_DIGEST_SIZE])
{
    ATCA_STATUS status;
    uint8_t root_hash[ATCA_SHA_DIGEST_SIZE];
    uint16_t cert_chain_size = ATCA_UINT16_HOST_TO_BE(2 + sizeof(root_hash) + mfg_cert_size + pu_cert_size);
    atcac_sha2_256_ctx ctx;

    do
    {
        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256(root_cert, root_cert_size, root_hash)))
        {
            printf("Calculate Root hash - atcac_sw_sha2_256 is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_init(&ctx)))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_init is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_update(&ctx, (uint8_t *)&cert_chain_size, sizeof(cert_chain_size))))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_update is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_update(&ctx, root_hash, sizeof(root_hash))))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_update is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_update(&ctx, mfg_cert, mfg_cert_size)))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_update is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_update(&ctx, pu_cert, pu_cert_size)))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_update is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_finish(&ctx, wpc_digest)))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_finish is failed with : %d\r\n", status);
            break;
        }
    } while (0);

    return status;
}

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************
ATCA_STATUS read_certificates(void)
{
    ATCA_STATUS status;

    if (ATCA_SUCCESS != (status = atcab_init(&ecc204_0_init_data)))
    {
        printf("atcab_init(ta010_cfg) is failed with an error %d\r\n", status);
        return status;
    }

    // read all bytes from slot 1... Since WPC PUC is almost full size of the slot, reading it all together
    if (ATCA_SUCCESS == (status = atcab_read_bytes_zone(ATCA_ZONE_DATA, 0x01, 0, g_product_cert, sizeof(g_product_cert))))
    {
        // Set correct cert size
        g_product_cert_size = (g_product_cert[2] * 256) + g_product_cert[3] + 4;
        print_hex("WPC Product Unit Certificate (PUC):", g_product_cert, g_product_cert_size);
    }
    else
    {
        printf("Reading Product unit certificate is failed with : %d\r\n", status);
    }

    return status;
}

ATCA_STATUS verify_certificate_chain(void)
{
    ATCA_STATUS status;

    do
    {
        if (ATCA_SUCCESS != (status = atcab_init(&atecc608_0_init_data)))
        {
            printf("atcab_init(ecc608_0) is failed with : %d\r\n", status);
            break;
        }

        // Verify Root certificate against root public key
        printf("Verify Root Certificate against its public key... ");
        if (ATCA_SUCCESS != (status = verify_certificate(root_cert, &root_cert[ROOT_PUBLIC_KEY_OFFSET], ROOT_SIGNATURE_OFFSET, ROOT_TBS_OFFSET, ROOT_TBS_SIZE)))
        {
            printf("Verification is failed with : %d\r\n", status);
            break;
        }
        printf("OK\r\n");

        // Verify Manufacturer certificate against root public key
        printf("Verify Mfg Certificate against root's public key... ");
        if (ATCA_SUCCESS != (status = verify_certificate(mfg_cert, &root_cert[ROOT_PUBLIC_KEY_OFFSET], MFG_SIGNATURE_OFFSET, MFG_TBS_OFFSET, MFG_TBS_SIZE)))
        {
            printf("\nVerification is failed with : %d\r\n", status);
            break;
        }
        printf("OK\r\n");

        // Verify Product Unit certificate against Mfg public key
        printf("Verify PU Certificate against root's public key... ");
        if (ATCA_SUCCESS != (status = verify_certificate(g_product_cert, &mfg_cert[MFG_PUBLIC_KEY_OFFSET], PUC_SIGNATURE_OFFSET, PUC_TBS_OFFSET, PUC_TBS_SIZE)))
        {
            printf("Verification is failed with : %d\r\n", status);
            break;
        }
        printf("OK\r\n");

        // Calculate Certificate chain digest from root cert, mfg cert, puc cert
        if (ATCA_SUCCESS != (status = calculate_wpc_digests(root_cert, sizeof(root_cert), mfg_cert, sizeof(mfg_cert), g_product_cert, g_product_cert_size, g_host_wpc_digest)))
        {
            printf("\nCert chain host digest - Calculating WPC chain digest is failed with : %d\r\n", status);
            return status;
        }
        print_hex("Certificate Chain (Host calculated digest):", g_host_wpc_digest, ATCA_SHA_DIGEST_SIZE);

    } while (0);

    return status;
}

ATCA_STATUS read_wpc_chain_digest(void)
{
    ATCA_STATUS status;

    do
    {
        if (ATCA_SUCCESS != (status = atcab_init(&ecc204_0_init_data)))
        {
            printf("atcab_init(ta010_cfg) is failed with : %d\r\n", status);
            break;
        }

        // read WPC digest from slot 2
        if (ATCA_SUCCESS != (status = atcab_read_bytes_zone(ATCA_ZONE_DATA, 0x02, 0, g_device_wpc_digest, sizeof(g_device_wpc_digest))))
        {
            printf("Cert chain device digest - atcab_read_bytes_zone is failed with: %d\r\n", status);
            break;
        }
        print_hex("Certificate Chain (Device digest):", g_device_wpc_digest, ATCA_SHA_DIGEST_SIZE);

    } while (0);

    return status;
}

ATCA_STATUS compare_wpc_chain_digests(void)
{
    uint8_t failed_at;

    printf("Comparing host and device chain digest values... ");
    if (0 != (failed_at = memcmp(g_host_wpc_digest, g_device_wpc_digest, ATCA_SHA_DIGEST_SIZE)))
    {
        printf("Failed at %d\r\n", failed_at);
    }
    printf("OK \r\n");

    return failed_at ? ATCA_CHECKMAC_VERIFY_FAILED : ATCA_SUCCESS;
}

ATCA_STATUS generate_challenge_to_power_transmitter(void)
{
    ATCA_STATUS status;

    do
    {
        if (ATCA_SUCCESS != (status = atcab_init(&atecc608_0_init_data)))
        {
            printf("PR - Generate challenge - atcab_init(ecc608_0) is failed with : %d\r\n", status);
            break;
        }

        // generate random challenge on host(Power Receiver)
        if (ATCA_SUCCESS != (status = atcacert_gen_challenge_hw(g_pr_challenge)))
        {
            printf("PR - Generate challenge - atcacert_gen_challenge_hw is failed with : %d\r\n", status);
            break;
        }
        print_hex("Power Receiver: Generated challenge:", g_pr_challenge, WPC_CHALLENGE_SIZE);

    } while (0);

    return status;
}

ATCA_STATUS verify_response_from_power_transmitter(void)
{
    ATCA_STATUS status;
    uint8_t pt_response[ATCA_SIG_SIZE];

    do
    {
        if (ATCA_SUCCESS != (status = atcab_init(&ecc204_0_init_data)))
        {
            printf("PT - Read response - atcab_init(ta010_cfg) is failed with : %d\r\n", status);
            break;
        }

        // get response from PT for Host Challenge
        if (ATCA_SUCCESS != (status = atcacert_get_response(0, g_pr_challenge, pt_response)))
        {
            printf("PT - Read response - atcacert_get_response is failed with : %d\r\n", status);
            break;
        }
        print_hex("Power Transmitter: Challenge response:", pt_response, ATCA_SIG_SIZE);

        if (ATCA_SUCCESS != (status = atcab_init(&atecc608_0_init_data)))
        {
            printf("PR - Verify response - atcab_init(ecc608_0) is failed with : %d\r\n", status);
            break;
        }

        printf("Power Receiver: Verify response: ");
        if (ATCA_SUCCESS != (status = atcacert_verify_response_hw(&g_product_cert[PUC_PUBLIC_KEY_OFFSET], g_pr_challenge, pt_response)))
        {
            printf("\nPR - Verify response - atcacert_verify_response_hw is failed with : %d\r\n", status);
            break;
        }
        printf("OK\r\n\n");

        printf("Power Receiver - Transmitter challenge response is verified!\r\n\r\n");
        printf("WPC Chain and Chain digest based authentication is successful!\r\n");

    } while (0);

    return status;
}

// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize(void)
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;
}

/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks(void)
{
    switch (appData.state)
    {
        case APP_STATE_INIT:
            appData.state = APP_STATE_READ_CERTIFICATE_CHAIN;
            break;

        case APP_STATE_READ_CERTIFICATE_CHAIN:
            appData.state = (read_certificates() == ATCA_SUCCESS) ? APP_STATE_VERIFY_CERTIFICATE_CHAIN : APP_STATE_FINISH;
            break;

        case APP_STATE_VERIFY_CERTIFICATE_CHAIN:
            appData.state = (verify_certificate_chain() == ATCA_SUCCESS) ? APP_STATE_READ_WPC_CHAIN_DIGEST : APP_STATE_FINISH;
            break;

        case APP_STATE_READ_WPC_CHAIN_DIGEST:
            appData.state = (read_wpc_chain_digest() == ATCA_SUCCESS) ? APP_STATE_VERIFY_WPC_CHAIN_DIGEST : APP_STATE_FINISH;
            break;

        case APP_STATE_VERIFY_WPC_CHAIN_DIGEST:
            appData.state = (compare_wpc_chain_digests() == ATCA_SUCCESS) ? APP_STATE_POWER_RECEIVER_GENERATE_CHALLENGE : APP_STATE_FINISH;
            break;

        case APP_STATE_POWER_RECEIVER_GENERATE_CHALLENGE:
            appData.state = (generate_challenge_to_power_transmitter() == ATCA_SUCCESS) ? APP_STATE_POWER_RECEIVER_VALIDATE_RESPONSE : APP_STATE_FINISH;
            break;

        case APP_STATE_POWER_RECEIVER_VALIDATE_RESPONSE:
            verify_response_from_power_transmitter();
            appData.state = APP_STATE_FINISH;
            break;

        case APP_STATE_FINISH:
            break;

        /* The default state should never be executed. */
        default:
            break;
    }
}

/*******************************************************************************
 End of File
 */

==> tpds_extension_ta010_support-1.6.1-py3-none-any.whl: wpc_auth_firmware/src/app.c <==
/*******************************************************************************
  MPLAB Harmony Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This file contains the source code for the MPLAB Harmony application.

  Description:
    This file contains the source code for the MPLAB Harmony application.  It
    implements the logic of the application's state machine and it may call
    API routines of other MPLAB Harmony modules in the system, such as drivers,
    system services, and middleware.  However, it does not call any of the
    system interfaces (such as the "Initialize" and "Tasks" functions) of any of
    the modules in the system or make any assumptions about when those functions
    are called.  That is the responsibility of the configuration-specific system
    files.
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include "app.h"
#include "cryptoauthlib.h"
#include "atcacert/atcacert_client.h"
#include "atcacert/atcacert_der.h"
#include "atcacert/atcacert_host_hw.h"
#include "../../../ta010_tflxwpc.h"

// *****************************************************************************
// *****************************************************************************
// Section: Global Data Definitions
// *****************************************************************************
// *****************************************************************************

#define WPC_CHALLENGE_SIZE 32

extern ATCAIfaceCfg ta010_0_init_data;
extern ATCAIfaceCfg atecc608_0_init_data;

uint8_t g_product_cert[320];
size_t g_product_cert_size;

uint8_t g_pr_challenge[WPC_CHALLENGE_SIZE];

uint8_t g_host_wpc_digest[ATCA_SHA_DIGEST_SIZE];
uint8_t g_device_wpc_digest[ATCA_SHA_DIGEST_SIZE];

// *****************************************************************************
/* Application Data

  Summary:
    Holds application data

  Description:
    This structure holds the application's data.

  Remarks:
    This structure should be initialized by the APP_Initialize function.

    Application strings and buffers are be defined outside this structure.
*/

APP_DATA appData;

// *****************************************************************************
// *****************************************************************************
// Section: Application Callback Functions
// *****************************************************************************
// *****************************************************************************

void print_hex(char *str, uint8_t *hex, size_t size)
{
    char display_str[1024];
    size_t display_len = sizeof(display_str);
    atcab_bin2hex(hex, size, display_str, &display_len);
    printf("\n%s\r\n%s\r\n\r\n", str, display_str);
}

ATCA_STATUS verify_certificate(const uint8_t *cert, const uint8_t *public_key, size_t sign_offset, size_t tbs_offset, size_t tbs_size)
{
    ATCA_STATUS status;
    uint8_t signature[ATCA_SIG_SIZE];
    uint8_t tbs_digest[ATCA_SHA_DIGEST_SIZE];
    size_t der_sig_size = sizeof(cert) - sign_offset;

    do
    {
        // Extract Signature from certificate
        if (ATCA_SUCCESS != (status = atcacert_der_dec_ecdsa_sig_value(&cert[sign_offset], &der_sig_size, signature)))
        {
            printf("atcacert_der_dec_ecdsa_sig_value is failed with : %d\r\n", status);
            break;
        }

        // calculate TBS digest
        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256(&cert[tbs_offset], tbs_size, tbs_digest)))
        {
            printf("Calculate TBS digest - atcac_sw_sha2_256 is failed with : %d\r\n", status);
            break;
        }

        // Verify Root certificate against root public key
        bool is_verified;
        if (ATCA_SUCCESS != (status = atcab_verify_extern(tbs_digest, signature, public_key, &is_verified)))
        {
            printf("atcab_verify_extern is failed with: %d\r\n", status);
            break;
        }
        status = is_verified ? ATCA_SUCCESS : ATCA_GEN_FAIL;

    } while (0);

    return status;
}

ATCA_STATUS calculate_wpc_digests(const uint8_t *root_cert, size_t root_cert_size, const uint8_t *mfg_cert, size_t mfg_cert_size,
                                  const uint8_t *pu_cert, size_t pu_cert_size, uint8_t wpc_digest[ATCA_SHA_DIGEST_SIZE])
{
    ATCA_STATUS status;
    uint8_t root_hash[ATCA_SHA_DIGEST_SIZE];
    uint16_t cert_chain_size = ATCA_UINT16_HOST_TO_BE(2 + sizeof(root_hash) + mfg_cert_size + pu_cert_size);
    atcac_sha2_256_ctx_t ctx;

    do
    {
        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256(root_cert, root_cert_size, root_hash)))
        {
            printf("Calculate Root hash - atcac_sw_sha2_256 is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_init(&ctx)))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_init is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_update(&ctx, (uint8_t *)&cert_chain_size, sizeof(cert_chain_size))))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_update is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_update(&ctx, root_hash, sizeof(root_hash))))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_update is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_update(&ctx, mfg_cert, mfg_cert_size)))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_update is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_update(&ctx, pu_cert, pu_cert_size)))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_update is failed with : %d\r\n", status);
            break;
        }

        if (ATCA_SUCCESS != (status = atcac_sw_sha2_256_finish(&ctx, wpc_digest)))
        {
            printf("Calc cert chain digest - atcac_sw_sha2_256_finish is failed with : %d\r\n", status);
            break;
        }
    } while (0);

    return status;
}

// *****************************************************************************
// *****************************************************************************
// Section: Application Local Functions
// *****************************************************************************
// *****************************************************************************

ATCA_STATUS read_certificates(void)
{
    ATCA_STATUS status;

    if (ATCA_SUCCESS != (status = atcab_init(&ta010_0_init_data)))
    {
        printf("atcab_init(ta010_cfg) is failed with an error %d\r\n", status);
        return status;
    }

    // read all bytes from slot 1... Since WPC PUC is almost full size of the slot, reading it all together
    if (ATCA_SUCCESS == (status = atcab_read_bytes_zone(ATCA_ZONE_DATA, 0x01, 0, g_product_cert, sizeof(g_product_cert))))
    {
        // Set correct cert size
        g_product_cert_size = (g_product_cert[2] * 256) + g_product_cert[3] + 4;
        print_hex("WPC Product Unit Certificate (PUC):", g_product_cert, g_product_cert_size);
    }
    else
    {
        printf("Reading Product unit certificate is failed with : %d\r\n", status);
    }

    return status;
}

ATCA_STATUS verify_certificate_chain(void)
{
    ATCA_STATUS status;

    do
    {
        if (ATCA_SUCCESS != (status = atcab_init(&atecc608_0_init_data)))
        {
            printf("atcab_init(ecc608_0) is failed with : %d\r\n", status);
            break;
        }

        // Verify Root certificate against root public key
        printf("Verify Root Certificate against its public key... ");
        if (ATCA_SUCCESS != (status = verify_certificate(root_cert, &root_cert[ROOT_PUBLIC_KEY_OFFSET], ROOT_SIGNATURE_OFFSET, ROOT_TBS_OFFSET, ROOT_TBS_SIZE)))
        {
            printf("Verification is failed with : %d\r\n", status);
            break;
        }
        printf("OK\r\n");

        // Verify Manufacturer certificate against root public key
        printf("Verify Mfg Certificate against root's public key... ");
        if (ATCA_SUCCESS != (status = verify_certificate(mfg_cert, &root_cert[ROOT_PUBLIC_KEY_OFFSET], MFG_SIGNATURE_OFFSET, MFG_TBS_OFFSET, MFG_TBS_SIZE)))
        {
            printf("\nVerification is failed with : %d\r\n", status);
            break;
        }
        printf("OK\r\n");

        // Verify Product Unit certificate against Mfg public key
        printf("Verify PU Certificate against root's public key... ");
        if (ATCA_SUCCESS != (status = verify_certificate(g_product_cert, &mfg_cert[MFG_PUBLIC_KEY_OFFSET], PUC_SIGNATURE_OFFSET, PUC_TBS_OFFSET, PUC_TBS_SIZE)))
        {
            printf("Verification is failed with : %d\r\n", status);
            break;
        }
        printf("OK\r\n");

        // Calculate Certificate chain digest from root cert, mfg cert, puc cert
        if (ATCA_SUCCESS != (status = calculate_wpc_digests(root_cert, sizeof(root_cert), mfg_cert, sizeof(mfg_cert), g_product_cert, g_product_cert_size, g_host_wpc_digest)))
        {
            printf("\nCert chain host digest - Calculating WPC chain digest is failed with : %d\r\n", status);
            return status;
        }
        print_hex("Certificate Chain (Host calculated digest):", g_host_wpc_digest, ATCA_SHA_DIGEST_SIZE);

    } while (0);

    return status;
}

ATCA_STATUS read_wpc_chain_digest(void)
{
    ATCA_STATUS status;

    do
    {
        if (ATCA_SUCCESS != (status = atcab_init(&ta010_0_init_data)))
        {
            printf("atcab_init(ta010_cfg) is failed with : %d\r\n", status);
            break;
        }

        // read WPC digest from slot 2
        if (ATCA_SUCCESS != (status = atcab_read_bytes_zone(ATCA_ZONE_DATA, 0x02, 0, g_device_wpc_digest, sizeof(g_device_wpc_digest))))
        {
            printf("Cert chain device digest - atcab_read_bytes_zone is failed with: %d\r\n", status);
            break;
        }
        print_hex("Certificate Chain (Device digest):", g_device_wpc_digest, ATCA_SHA_DIGEST_SIZE);

    } while (0);

    return status;
}

ATCA_STATUS compare_wpc_chain_digests(void)
{
    uint8_t failed_at;

    printf("Comparing host and device chain digest values... ");
    if (0 != (failed_at = memcmp(g_host_wpc_digest, g_device_wpc_digest, ATCA_SHA_DIGEST_SIZE)))
    {
        printf("Failed at %d\r\n", failed_at);
    }
    printf("OK \r\n");

    return failed_at ? ATCA_CHECKMAC_VERIFY_FAILED : ATCA_SUCCESS;
}

ATCA_STATUS generate_challenge_to_power_transmitter(void)
{
    ATCA_STATUS status;

    do
    {
        if (ATCA_SUCCESS != (status = atcab_init(&atecc608_0_init_data)))
        {
            printf("PR - Generate challenge - atcab_init(ecc608_0) is failed with : %d\r\n", status);
            break;
        }

        // generate random challenge on host(Power Receiver)
        if (ATCA_SUCCESS != (status = atcacert_gen_challenge_hw(g_pr_challenge)))
        {
            printf("PR - Generate challenge - atcacert_gen_challenge_hw is failed with : %d\r\n", status);
            break;
        }
        print_hex("Power Receiver: Generated challenge:", g_pr_challenge, WPC_CHALLENGE_SIZE);

    } while (0);

    return status;
}

ATCA_STATUS verify_response_from_power_transmitter(void)
{
    ATCA_STATUS status;
    uint8_t pt_response[ATCA_SIG_SIZE];

    do
    {
        if (ATCA_SUCCESS != (status = atcab_init(&ta010_0_init_data)))
        {
            printf("PT - Read response - atcab_init(ta010_cfg) is failed with : %d\r\n", status);
            break;
        }

        // get response from PT for Host Challenge
        if (ATCA_SUCCESS != (status = atcacert_get_response(0, g_pr_challenge, pt_response)))
        {
            printf("PT - Read response - atcacert_get_response is failed with : %d\r\n", status);
            break;
        }
        print_hex("Power Transmitter: Challenge response:", pt_response, ATCA_SIG_SIZE);

        if (ATCA_SUCCESS != (status = atcab_init(&atecc608_0_init_data)))
        {
            printf("PR - Verify response - atcab_init(ecc608_0) is failed with : %d\r\n", status);
            break;
        }

        printf("Power Receiver: Verify response: ");
        if (ATCA_SUCCESS != (status = atcacert_verify_response_hw(&g_product_cert[PUC_PUBLIC_KEY_OFFSET], g_pr_challenge, pt_response)))
        {
            printf("\nPR - Verify response - atcacert_verify_response_hw is failed with : %d\r\n", status);
            break;
        }
        printf("OK\r\n\n");

        printf("Power Receiver - Transmitter challenge response is verified!\r\n\r\n");
        printf("WPC Chain and Chain digest based authentication is successful!\r\n");

    } while (0);

    return status;
}


// *****************************************************************************
// *****************************************************************************
// Section: Application Initialization and State Machine Functions
// *****************************************************************************
// *****************************************************************************

/*******************************************************************************
  Function:
    void APP_Initialize ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Initialize ( void )
{
    /* Place the App state machine in its initial state. */
    appData.state = APP_STATE_INIT;



    /* TODO: Initialize your application's state machine and other
     * parameters.
     */
}


/******************************************************************************
  Function:
    void APP_Tasks ( void )

  Remarks:
    See prototype in app.h.
 */

void APP_Tasks ( void )
{
    ATCA_STATUS status;

    /* Check the application's current state. */
    switch ( appData.state )
    {
        /* Application's initial state. */
        case APP_STATE_INIT:
            appData.state = APP_STATE_READ_CERTIFICATE_CHAIN;
            break;

        case APP_STATE_READ_CERTIFICATE_CHAIN:
            status = read_certificates();
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_VERIFY_CERTIFICATE_CHAIN : APP_STATE_FINISH;
            break;

        case APP_STATE_VERIFY_CERTIFICATE_CHAIN:
            status = verify_certificate_chain();
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_READ_WPC_CHAIN_DIGEST : APP_STATE_FINISH;
            break;

        case APP_STATE_READ_WPC_CHAIN_DIGEST:
            status = read_wpc_chain_digest();
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_VERIFY_WPC_CHAIN_DIGEST : APP_STATE_FINISH;
            break;

        case APP_STATE_VERIFY_WPC_CHAIN_DIGEST:
            status = compare_wpc_chain_digests();
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_POWER_RECEIVER_GENERATE_CHALLENGE : APP_STATE_FINISH;
            break;

        case APP_STATE_POWER_RECEIVER_GENERATE_CHALLENGE:
            status = generate_challenge_to_power_transmitter();
            appData.state = (ATCA_SUCCESS == status) ? APP_STATE_POWER_RECEIVER_VALIDATE_RESPONSE : APP_STATE_FINISH;
            break;

        case APP_STATE_POWER_RECEIVER_VALIDATE_RESPONSE:
            verify_response_from_power_transmitter();
            appData.state = APP_STATE_FINISH;
            break;

        case APP_STATE_FINISH:
            break;

        /* The default state should never be executed. */
        default:
        {
            break;
        }
    }
}


/*******************************************************************************
 End of File
 */
